@@ -87,14 +87,11 @@ static void stream_state_changed(struct avdtp_stream *stream,
 	switch (new_state) {
 	case AVDTP_STATE_IDLE:
 		value = FALSE;
-		g_dbus_emit_signal(dev->conn, dev->path,
-						AUDIO_SINK_INTERFACE,
-						"Disconnected",
-						DBUS_TYPE_INVALID);
-		dbus_connection_emit_property_changed(dev->conn, dev->path,
-						AUDIO_SINK_INTERFACE,
-						"Connected",
-						DBUS_TYPE_BOOLEAN, &value);
+		g_dbus_emit_signal(dev->conn, dev->path, AUDIO_SINK_INTERFACE,
+					"Disconnected", DBUS_TYPE_INVALID);
+		emit_property_changed(dev->conn, dev->path,
+					AUDIO_SINK_INTERFACE, "Connected",
+					DBUS_TYPE_BOOLEAN, &value);
 		if (sink->disconnect) {
 			DBusMessage *reply;
 			struct pending_request *p;
@@ -103,8 +100,7 @@ static void stream_state_changed(struct avdtp_stream *stream,
 			sink->disconnect = NULL;
 
 			reply = dbus_message_new_method_return(p->msg);
-			dbus_connection_send(p->conn, reply, NULL);
-			dbus_message_unref(reply);
+			g_dbus_send_message(p->conn, reply);
 			pending_request_free(p);
 		}
 
@@ -119,39 +115,32 @@ static void stream_state_changed(struct avdtp_stream *stream,
 		if (old_state == AVDTP_STATE_CONFIGURED) {
 			value = TRUE;
 			g_dbus_emit_signal(dev->conn, dev->path,
-							AUDIO_SINK_INTERFACE,
-							"Connected",
-							DBUS_TYPE_INVALID);
-			dbus_connection_emit_property_changed(dev->conn,
-							dev->path,
-							AUDIO_SINK_INTERFACE,
-							"Connected",
-							DBUS_TYPE_BOOLEAN,
-							&value);
+						AUDIO_SINK_INTERFACE,
+						"Connected",
+						DBUS_TYPE_INVALID);
+			emit_property_changed(dev->conn, dev->path,
+						AUDIO_SINK_INTERFACE,
+						"Connected",
+						DBUS_TYPE_BOOLEAN, &value);
 		} else if (old_state == AVDTP_STATE_STREAMING) {
 			value = FALSE;
 			g_dbus_emit_signal(dev->conn, dev->path,
-							AUDIO_SINK_INTERFACE,
-							"Stopped",
-							DBUS_TYPE_INVALID);
-			dbus_connection_emit_property_changed(dev->conn,
-							dev->path,
-							AUDIO_SINK_INTERFACE,
-							"Playing",
-							DBUS_TYPE_BOOLEAN,
-							&value);
+						AUDIO_SINK_INTERFACE,
+						"Stopped",
+						DBUS_TYPE_INVALID);
+			emit_property_changed(dev->conn, dev->path,
+						AUDIO_SINK_INTERFACE,
+						"Playing",
+						DBUS_TYPE_BOOLEAN, &value);
 		}
 		break;
 	case AVDTP_STATE_STREAMING:
 		value = TRUE;
-		g_dbus_emit_signal(dev->conn, dev->path,
-						AUDIO_SINK_INTERFACE,
-						"Playing",
-						DBUS_TYPE_INVALID);
-		dbus_connection_emit_property_changed(dev->conn, dev->path,
-						AUDIO_SINK_INTERFACE,
-						"Playing",
-						DBUS_TYPE_BOOLEAN, &value);
+		g_dbus_emit_signal(dev->conn, dev->path, AUDIO_SINK_INTERFACE,
+					"Playing", DBUS_TYPE_INVALID);
+		emit_property_changed(dev->conn, dev->path,
+					AUDIO_SINK_INTERFACE, "Playing",
+					DBUS_TYPE_BOOLEAN, &value);
 		break;
 	case AVDTP_STATE_CONFIGURED:
 	case AVDTP_STATE_CLOSING:
@@ -178,8 +167,7 @@ static gboolean stream_setup_retry(gpointer user_data)
 		DBusMessage *reply;
 		debug("Stream successfully created, after XCASE connect:connect");
 		reply = dbus_message_new_method_return(pending->msg);
-		dbus_connection_send(pending->conn, reply, NULL);
-		dbus_message_unref(reply);
+		g_dbus_send_message(pending->conn, reply);
 	} else {
 		debug("Stream setup failed, after XCASE connect:connect");
 		error_failed(pending->conn, pending->msg, "Stream setup failed");
@@ -204,8 +192,7 @@ static void stream_setup_complete(struct avdtp *session, struct a2dp_sep *sep,
 		DBusMessage *reply;
 		sink->connect = NULL;
 		reply = dbus_message_new_method_return(pending->msg);
-		dbus_connection_send(pending->conn, reply, NULL);
-		dbus_message_unref(reply);
+		g_dbus_send_message(pending->conn, reply);
 		pending_request_free(pending);
 		debug("Stream successfully created");
 	} else {
@@ -519,6 +506,10 @@ static DBusMessage *sink_get_properties(DBusConnection *conn,
 	DBusMessageIter dict;
 	gboolean value;
 
+	reply = dbus_message_new_method_return(msg);
+	if (!reply)
+		return NULL;
+
 	dbus_message_iter_init_append(reply, &iter);
 
 	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
@@ -528,13 +519,11 @@ static DBusMessage *sink_get_properties(DBusConnection *conn,
 
 	/* Playing */
 	value = (device->sink->state == AVDTP_STATE_STREAMING);
-	dbus_message_iter_append_dict_entry(&dict, "Playing",
-						DBUS_TYPE_BOOLEAN, &value);
+	dict_append_entry(&dict, "Playing", DBUS_TYPE_BOOLEAN, &value);
 
 	/* Connected */
 	value = (device->sink->state >= AVDTP_STATE_CONFIGURED);
-	dbus_message_iter_append_dict_entry(&dict, "Connected",
-						DBUS_TYPE_BOOLEAN, &value);
+	dict_append_entry(&dict, "Connected", DBUS_TYPE_BOOLEAN, &value);
 
 	dbus_message_iter_close_container(&iter, &dict);
 
