@@ -33,41 +33,10 @@
 #include <stdint.h>
 #include <string.h>
 #include <getopt.h>
-#include <byteswap.h>
 #include <sys/stat.h>
 
 #include "sbc.h"
-
-#if __BYTE_ORDER == __LITTLE_ENDIAN
-#define COMPOSE_ID(a,b,c,d)	((a) | ((b)<<8) | ((c)<<16) | ((d)<<24))
-#define LE_SHORT(v)		(v)
-#define LE_INT(v)		(v)
-#define BE_SHORT(v)		bswap_16(v)
-#define BE_INT(v)		bswap_32(v)
-#elif __BYTE_ORDER == __BIG_ENDIAN
-#define COMPOSE_ID(a,b,c,d)	((d) | ((c)<<8) | ((b)<<16) | ((a)<<24))
-#define LE_SHORT(v)		bswap_16(v)
-#define LE_INT(v)		bswap_32(v)
-#define BE_SHORT(v)		(v)
-#define BE_INT(v)		(v)
-#else
-#error "Wrong endian"
-#endif
-
-#define AU_MAGIC		COMPOSE_ID('.','s','n','d')
-
-#define AU_FMT_ULAW		1
-#define AU_FMT_LIN8		2
-#define AU_FMT_LIN16		3
-
-struct au_header {
-	uint32_t magic;		/* '.snd' */
-	uint32_t hdr_size;	/* size of header (min 24) */
-	uint32_t data_size;	/* size of data */
-	uint32_t encoding;	/* see to AU_FMT_XXXX */
-	uint32_t sample_rate;	/* sample rate */
-	uint32_t channels;	/* number of channels (voices) */
-};
+#include "formats.h"
 
 static ssize_t __read(int fd, void *buf, size_t count)
 {
@@ -101,7 +70,7 @@ static ssize_t __write(int fd, const void *buf, size_t count)
 	return pos;
 }
 
-static void encode(char *filename, int subbands, int joint)
+static void encode(char *filename, int subbands, int bitpool, int joint)
 {
 	struct au_header *au_hdr;
 	unsigned char input[2048], output[2048];
@@ -169,10 +138,12 @@ static void encode(char *filename, int subbands, int joint)
 	size = len - BE_INT(au_hdr->hdr_size);
 	memmove(input, input + BE_INT(au_hdr->hdr_size), size);
 
+	sbc.bitpool = bitpool;
+
 	while (1) {
 		if (size < sizeof(input)) {
 			len = __read(fd, input + size, sizeof(input) - size);
-			if (len == 0)
+			if (len == 0 && size == 0)
 				break;
 
 			if (len < 0) {
@@ -183,8 +154,8 @@ static void encode(char *filename, int subbands, int joint)
 			size += len;
 		}
 
-		len = sbc_encode(&sbc, input, size, output, sizeof(output),
-					&encoded);
+		len = sbc_encode(&sbc, input, size,
+					output, sizeof(output), &encoded);
 		if (len < size)
 			memmove(input, input + len, size - len);
 
@@ -220,6 +191,7 @@ static void usage(void)
 		"\t-h, --help           Display help\n"
 		"\t-v, --verbose        Verbose mode\n"
 		"\t-s, --subbands       Number of subbands to use (4 or 8)\n"
+		"\t-b, --bitpool        Bitpool value (default is 32)\n"
 		"\t-j, --joint          Joint stereo\n"
 		"\n");
 }
@@ -228,15 +200,16 @@ static struct option main_options[] = {
 	{ "help",	0, 0, 'h' },
 	{ "verbose",	0, 0, 'v' },
 	{ "subbands",	1, 0, 's' },
+	{ "bitpool",	1, 0, 'b' },
 	{ "joint",	0, 0, 'j' },
 	{ 0, 0, 0, 0 }
 };
 
 int main(int argc, char *argv[])
 {
-	int i, opt, verbose = 0, subbands = 8, joint = 0;
+	int i, opt, verbose = 0, subbands = 8, bitpool = 32, joint = 0;
 
-	while ((opt = getopt_long(argc, argv, "+hvs:j", main_options, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "+hvs:b:j", main_options, NULL)) != -1) {
 		switch(opt) {
 		case 'h':
 			usage();
@@ -247,7 +220,7 @@ int main(int argc, char *argv[])
 			break;
 
 		case 's':
-			subbands = atoi(strdup(optarg));
+			subbands = atoi(optarg);
 			if (subbands != 8 && subbands != 4) {
 				fprintf(stderr, "Invalid subbands %d!\n",
 						subbands);
@@ -255,6 +228,10 @@ int main(int argc, char *argv[])
 			}
 			break;
 
+		case 'b':
+			bitpool = atoi(optarg);
+			break;
+
 		case 'j':
 			joint = 1;
 			break;
@@ -274,7 +251,7 @@ int main(int argc, char *argv[])
 	}
 
 	for (i = 0; i < argc; i++)
-		encode(argv[i], subbands, joint);
+		encode(argv[i], subbands, bitpool, joint);
 
 	return 0;
 }
