@@ -700,6 +700,13 @@ static void session_free(struct session_req *req)
 	g_free(req);
 }
 
+static void session_owner_exit(DBusConnection *conn, void *user_data)
+{
+	struct session_req *req = user_data;
+
+	session_free(req);
+}
+
 static struct session_req *session_ref(struct session_req *req)
 {
 	req->refcount++;
@@ -755,14 +762,12 @@ static void confirm_mode_cb(struct agent *agent, DBusError *err, void *data)
 
 	if (err && dbus_error_is_set(err)) {
 		reply = dbus_message_new_error(req->msg, err->name, err->message);
-		dbus_connection_send(req->conn, reply, NULL);
-		dbus_message_unref(reply);
+		g_dbus_send_message(req->conn, reply);
 		goto cleanup;
 	}
 
 	reply = set_mode(req->conn, req->msg, req->mode, req->adapter);
-	dbus_connection_send(req->conn, reply, NULL);
-	dbus_message_unref(reply);
+	g_dbus_send_message(req->conn, reply);
 
 	if (!find_session(req->adapter->mode_sessions, req->msg))
 		goto cleanup;
@@ -916,10 +921,8 @@ static void reply_authentication_failure(struct bonding_request_info *bonding)
 			bonding->hci_status : HCI_AUTHENTICATION_FAILURE;
 
 	reply = new_authentication_return(bonding->msg, status);
-	if (reply) {
-		dbus_connection_send(bonding->conn, reply, NULL);
-		dbus_message_unref(reply);
-	}
+	if (reply)
+		g_dbus_send_message(bonding->conn, reply);
 }
 
 struct btd_device *adapter_find_device(struct btd_adapter *adapter, const char *dest)
@@ -1515,7 +1518,7 @@ static DBusMessage *adapter_start_discovery(DBusConnection *conn,
 
 done:
 	req = create_session(adapter, conn, msg, 0,
-			(GDBusWatchFunction) session_free);
+				session_owner_exit);
 
 	adapter->disc_sessions = g_slist_append(adapter->disc_sessions, req);
 
@@ -1737,7 +1740,7 @@ static DBusMessage *mode_request(DBusConnection *conn,
 	req = find_session(adapter->mode_sessions, msg);
 	if (!req) {
 		req = create_session(adapter, conn, msg, new_mode,
-					(GDBusWatchFunction) session_free);
+					session_owner_exit);
 		adapter->mode_sessions = g_slist_append(adapter->mode_sessions,
 					req);
 	} else {
@@ -2678,8 +2681,6 @@ int adapter_stop(struct btd_adapter *adapter)
 	adapter->mode = MODE_OFF;
 	adapter->state = DISCOVER_TYPE_NONE;
 
-	unload_drivers(adapter);
-
 	info("Adapter %s has been disabled", adapter->path);
 
 	return 0;
@@ -2791,6 +2792,8 @@ void adapter_remove(struct btd_adapter *adapter)
 
 	debug("Removing adapter %s", path);
 
+	unload_drivers(adapter);
+
 	for (l = adapter->devices; l; l = l->next)
 		device_remove(connection, l->data);
 	g_slist_free(adapter->devices);
