@@ -87,6 +87,8 @@ struct btd_device {
 
 	/* Whether were creating a security mode 3 connection */
 	gboolean	secmode3;
+
+	sdp_list_t	*tmp_records;
 };
 
 struct browse_req {
@@ -134,6 +136,9 @@ static void device_free(gpointer user_data)
 	if (device->disconn_timer)
 		g_source_remove(device->disconn_timer);
 
+	if (device->discov_timer)
+		g_source_remove(device->discov_timer);
+
 	g_free(device->path);
 	g_free(device);
 }
@@ -192,8 +197,7 @@ static DBusMessage *get_properties(DBusConnection *conn,
 
 	/* Address */
 	ptr = dstaddr;
-	dbus_message_iter_append_dict_entry(&dict, "Address",
-						DBUS_TYPE_STRING, &ptr);
+	dict_append_entry(&dict, "Address", DBUS_TYPE_STRING, &ptr);
 
 	/* Name */
 	ptr = NULL;
@@ -203,8 +207,7 @@ static DBusMessage *get_properties(DBusConnection *conn,
 
 	if (read_device_name(srcaddr, dstaddr, name) == 0) {
 		ptr = name;
-		dbus_message_iter_append_dict_entry(&dict, "Name",
-						DBUS_TYPE_STRING, &ptr);
+		dict_append_entry(&dict, "Name", DBUS_TYPE_STRING, &ptr);
 	}
 
 	/* Alias (fallback to name or address) */
@@ -217,30 +220,26 @@ static DBusMessage *get_properties(DBusConnection *conn,
 		ptr = name;
 
 	if (ptr)
-		dbus_message_iter_append_dict_entry(&dict, "Alias",
-						DBUS_TYPE_STRING, &ptr);
+		dict_append_entry(&dict, "Alias", DBUS_TYPE_STRING, &ptr);
 
 	/* Class */
 	if (read_remote_class(&src, &device->bdaddr, &class) == 0) {
 		const char *icon = class_to_icon(class);
 
-		dbus_message_iter_append_dict_entry(&dict, "Class",
-						DBUS_TYPE_UINT32, &class);
+		dict_append_entry(&dict, "Class", DBUS_TYPE_UINT32, &class);
 
 		if (icon)
-			dbus_message_iter_append_dict_entry(&dict, "Icon",
+			dict_append_entry(&dict, "Icon",
 						DBUS_TYPE_STRING, &icon);
 	}
 
 	/* Paired */
 	boolean = device_is_paired(device);
-	dbus_message_iter_append_dict_entry(&dict, "Paired",
-						DBUS_TYPE_BOOLEAN, &boolean);
+	dict_append_entry(&dict, "Paired", DBUS_TYPE_BOOLEAN, &boolean);
 
 	/* Trusted */
 	boolean = read_trust(&src, dstaddr, GLOBAL_TRUST);
-	dbus_message_iter_append_dict_entry(&dict, "Trusted",
-						DBUS_TYPE_BOOLEAN, &boolean);
+	dict_append_entry(&dict, "Trusted", DBUS_TYPE_BOOLEAN, &boolean);
 
 	/* Connected */
 	dev = adapter_search_active_conn_by_bdaddr(adapter, &device->bdaddr);
@@ -249,21 +248,18 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	else
 		boolean = FALSE;
 
-	dbus_message_iter_append_dict_entry(&dict, "Connected",
-						DBUS_TYPE_BOOLEAN, &boolean);
+	dict_append_entry(&dict, "Connected", DBUS_TYPE_BOOLEAN, &boolean);
 
 	/* UUIDs */
 	uuids = g_new0(char *, g_slist_length(device->uuids) + 1);
 	for (i = 0, l = device->uuids; l; l = l->next, i++)
 		uuids[i] = l->data;
-	dbus_message_iter_append_dict_entry(&dict, "UUIDs",
-						DBUS_TYPE_ARRAY, &uuids);
+	dict_append_array(&dict, "UUIDs", DBUS_TYPE_STRING, &uuids, i);
 	g_free(uuids);
 
 	/* Adapter */
 	ptr = adapter_get_path(adapter);
-	dbus_message_iter_append_dict_entry(&dict, "Adapter",
-						DBUS_TYPE_OBJECT_PATH, &ptr);
+	dict_append_entry(&dict, "Adapter", DBUS_TYPE_OBJECT_PATH, &ptr);
 
 	dbus_message_iter_close_container(&iter, &dict);
 
@@ -291,9 +287,9 @@ static DBusMessage *set_alias(DBusConnection *conn, DBusMessage *msg,
 				ERROR_INTERFACE ".Failed",
 				strerror(-err));
 
-	dbus_connection_emit_property_changed(conn, dbus_message_get_path(msg),
-					DEVICE_INTERFACE, "Alias",
-					DBUS_TYPE_STRING, &alias);
+	emit_property_changed(conn, dbus_message_get_path(msg),
+				DEVICE_INTERFACE, "Alias",
+				DBUS_TYPE_STRING, &alias);
 
 	return dbus_message_new_method_return(msg);
 }
@@ -313,9 +309,9 @@ static DBusMessage *set_trust(DBusConnection *conn, DBusMessage *msg,
 
 	write_trust(srcaddr, dstaddr, GLOBAL_TRUST, value);
 
-	dbus_connection_emit_property_changed(conn, dbus_message_get_path(msg),
-					DEVICE_INTERFACE, "Trusted",
-					DBUS_TYPE_BOOLEAN, &value);
+	emit_property_changed(conn, dbus_message_get_path(msg),
+				DEVICE_INTERFACE, "Trusted",
+				DBUS_TYPE_BOOLEAN, &value);
 
 	return dbus_message_new_method_return(msg);
 }
@@ -368,7 +364,7 @@ static DBusMessage *set_property(DBusConnection *conn,
 	return invalid_args(msg);
 }
 
-static void discover_services_req_exit(void *user_data)
+static void discover_services_req_exit(DBusConnection *conn, void *user_data)
 {
 	struct btd_device *device = user_data;
 	struct btd_adapter *adapter = device->adapter;
@@ -397,7 +393,7 @@ static DBusMessage *discover_services(DBusConnection *conn,
 		goto fail;
 
 	if (strlen(pattern) == 0) {
-		err = device_browse(device, conn, msg, NULL);
+		err = device_browse(device, conn, msg, NULL, FALSE);
 		if (err < 0)
 			goto fail;
 	} else {
@@ -406,7 +402,7 @@ static DBusMessage *discover_services(DBusConnection *conn,
 		if (bt_string2uuid(&uuid, pattern) < 0)
 			return invalid_args(msg);
 
-		err = device_browse(device, conn, msg, &uuid);
+		err = device_browse(device, conn, msg, &uuid, FALSE);
 		if (err < 0)
 			goto fail;
 	}
@@ -530,15 +526,8 @@ void device_set_connected(DBusConnection *conn, struct btd_device *device,
 {
 	device->connected = connected;
 
-	if (!connected && device->discov_timer) {
-		g_source_remove(device->discov_timer);
-		device->discov_timer = 0;
-	}
-
-	dbus_connection_emit_property_changed(conn, device->path,
-						DEVICE_INTERFACE,
-						"Connected", DBUS_TYPE_BOOLEAN,
-						&connected);
+	emit_property_changed(conn, device->path, DEVICE_INTERFACE,
+				"Connected", DBUS_TYPE_BOOLEAN, &connected);
 
 	if (connected && device->secmode3) {
 		struct btd_adapter *adapter = device_get_adapter(device);
@@ -618,7 +607,7 @@ gint device_address_cmp(struct btd_device *device, const gchar *address)
 	return strcasecmp(addr, address);
 }
 
-void device_probe_drivers(struct btd_device *device, GSList *uuids, sdp_list_t *recs)
+void device_probe_drivers(struct btd_device *device, GSList *uuids)
 {
 	GSList *list;
 	const char **uuid;
@@ -628,69 +617,80 @@ void device_probe_drivers(struct btd_device *device, GSList *uuids, sdp_list_t *
 
 	for (list = device_drivers; list; list = list->next) {
 		struct btd_device_driver *driver = list->data;
-		GSList *records = NULL;
+		GSList *probe_uuids = NULL;
 
 		for (uuid = driver->uuids; *uuid; uuid++) {
-			sdp_record_t *rec;
+			GSList *match;
 
-			if (!g_slist_find_custom(uuids, *uuid,
-					(GCompareFunc) strcasecmp))
+			match = g_slist_find_custom(uuids, *uuid,
+							(GCompareFunc) strcasecmp);
+			if (!match)
 				continue;
 
-			rec = find_record_in_list(recs, *uuid);
-			if (!rec)
-				continue;
-
-			records = g_slist_append(records, rec);
+			probe_uuids = g_slist_append(probe_uuids, match->data);
 		}
 
-		if (records) {
+		if (probe_uuids) {
 			struct btd_driver_data *driver_data = g_new0(struct btd_driver_data, 1);
 
-			err = driver->probe(device, records);
+			err = driver->probe(device, probe_uuids);
 			if (err < 0) {
-				error("probe failed for driver %s",
-							driver->name);
+				error("probe failed with driver %s for device %s",
+							driver->name, device->path);
 
 				g_free(driver_data);
+				g_slist_free(probe_uuids);
 				continue;
 			}
 
 			driver_data->driver = driver;
 			device->drivers = g_slist_append(device->drivers,
 								driver_data);
+			g_slist_free(probe_uuids);
 		}
 	}
 
 	for (list = uuids; list; list = list->next) {
 		GSList *l = g_slist_find_custom(device->uuids, list->data,
-							(GCompareFunc) strcmp);
+							(GCompareFunc) strcasecmp);
 		if (l)
 			continue;
 
 		device->uuids = g_slist_insert_sorted(device->uuids,
-					list->data, (GCompareFunc) strcmp);
+							g_strdup(list->data),
+							(GCompareFunc) strcasecmp);
+	}
+
+	if (device->tmp_records) {
+		sdp_list_free(device->tmp_records,
+				(sdp_free_func_t) sdp_record_free);
+		device->tmp_records = NULL;
 	}
 }
 
-void device_remove_drivers(struct btd_device *device, GSList *uuids, sdp_list_t *recs)
+void device_remove_drivers(struct btd_device *device, GSList *uuids)
 {
 	struct btd_adapter *adapter = device_get_adapter(device);
-	GSList *list;
+	GSList *list, *next;
 	char srcaddr[18], dstaddr[18];
 	bdaddr_t src;
+	sdp_list_t *records;
 
 	adapter_get_address(adapter, &src);
 	ba2str(&src, srcaddr);
 	ba2str(&device->bdaddr, dstaddr);
 
+	records = read_records(&src, &device->bdaddr);
+
 	debug("Remove drivers for %s", device->path);
 
-	for (list = device->drivers; list; list = list->next) {
+	for (list = device->drivers; list; list = next) {
 		struct btd_driver_data *driver_data = list->data;
 		struct btd_device_driver *driver = driver_data->driver;
 		const char **uuid;
 
+		next = list->next;
+
 		for (uuid = driver->uuids; *uuid; uuid++) {
 			sdp_record_t *rec;
 
@@ -698,20 +698,29 @@ void device_remove_drivers(struct btd_device *device, GSList *uuids, sdp_list_t
 					(GCompareFunc) strcasecmp))
 				continue;
 
+			debug("UUID %s was removed from device %s", dstaddr);
+
 			driver->remove(device);
 			device->drivers = g_slist_remove(device->drivers,
 								driver_data);
 
 			g_free(driver_data);
 
-			rec = find_record_in_list(recs, *uuid);
+
+			rec = find_record_in_list(records, *uuid);
 			if (!rec)
 				continue;
 
 			delete_record(srcaddr, dstaddr, rec->handle);
+
+			records = sdp_list_remove(records, rec);
+			sdp_record_free(rec);
 		}
 	}
 
+	if (records)
+		sdp_list_free(records, (sdp_free_func_t) sdp_record_free);
+
 	for (list = uuids; list; list = list->next)
 		device->uuids = g_slist_remove(device->uuids, list->data);
 }
@@ -804,14 +813,12 @@ static void discover_device_reply(struct browse_req *req, sdp_list_t *recs)
 
 	dbus_message_iter_close_container(&iter, &dict);
 
-	dbus_connection_send(req->conn, reply, NULL);
-	dbus_message_unref(reply);
+	g_dbus_send_message(req->conn, reply);
 }
 
-static void services_changed(struct browse_req *req)
+static void services_changed(struct btd_device *device)
 {
 	DBusConnection *conn = get_dbus_connection();
-	struct btd_device *device = req->device;
 	char **uuids;
 	GSList *l;
 	int i;
@@ -820,9 +827,8 @@ static void services_changed(struct browse_req *req)
 	for (i = 0, l = device->uuids; l; l = l->next, i++)
 		uuids[i] = l->data;
 
-	dbus_connection_emit_property_changed(conn, device->path,
-					DEVICE_INTERFACE, "UUIDs",
-					DBUS_TYPE_ARRAY, &uuids);
+	emit_array_property_changed(conn, device->path, DEVICE_INTERFACE,
+					"UUIDs", DBUS_TYPE_STRING, &uuids);
 
 	g_free(uuids);
 }
@@ -933,7 +939,7 @@ static void update_services(struct browse_req *req, sdp_list_t *recs)
 	}
 }
 
-static void store(struct btd_device *device)
+static void store_profiles(struct btd_device *device)
 {
 	struct btd_adapter *adapter = device->adapter;
 	bdaddr_t src;
@@ -951,6 +957,31 @@ static void store(struct btd_device *device)
 	g_free(str);
 }
 
+static void browse_req_free(struct browse_req *req)
+{
+	struct btd_device *device = req->device;
+
+	device->discov_active = 0;
+
+	if (device->discov_requestor) {
+		g_dbus_remove_watch(req->conn, device->discov_listener);
+		device->discov_listener = 0;
+		g_free(device->discov_requestor);
+		device->discov_requestor = NULL;
+	}
+
+	if (req->msg)
+		dbus_message_unref(req->msg);
+	if (req->conn)
+		dbus_connection_unref(req->conn);
+	g_slist_foreach(req->uuids_added, (GFunc) g_free, NULL);
+	g_slist_free(req->uuids_added);
+	g_slist_free(req->uuids_removed);
+	if (req->records)
+		sdp_list_free(req->records, (sdp_free_func_t) sdp_record_free);
+	g_free(req);
+}
+
 static void search_cb(sdp_list_t *recs, int err, gpointer user_data)
 {
 	struct browse_req *req = user_data;
@@ -970,20 +1001,26 @@ static void search_cb(sdp_list_t *recs, int err, gpointer user_data)
 		goto proceed;
 	}
 
+	if (device->tmp_records && req->records) {
+		sdp_list_free(device->tmp_records, (sdp_free_func_t) sdp_record_free);
+		device->tmp_records = req->records;
+		req->records = NULL;
+	}
+
 	/* Probe matching drivers for services added */
 	if (req->uuids_added)
-		device_probe_drivers(device, req->uuids_added, req->records);
+		device_probe_drivers(device, req->uuids_added);
 
 	/* Remove drivers for services removed */
 	if (req->uuids_removed)
-		device_remove_drivers(device, req->uuids_removed, req->records);
+		device_remove_drivers(device, req->uuids_removed);
 
 	/* Propagate services changes */
-	services_changed(req);
+	services_changed(req->device);
 
 proceed:
 	/* Store the device's profiles in the filesystem */
-	store(device);
+	store_profiles(device);
 
 	if (req->msg) {
 		if (dbus_message_is_method_call(req->msg, DEVICE_INTERFACE,
@@ -1012,29 +1049,10 @@ proceed:
 	dbus_message_append_args(reply, DBUS_TYPE_OBJECT_PATH, &device->path,
 							DBUS_TYPE_INVALID);
 
-	dbus_connection_send(req->conn, reply, NULL);
-	dbus_message_unref(reply);
+	g_dbus_send_message(req->conn, reply);
 
 cleanup:
-	device->discov_active = 0;
-
-	if (device->discov_requestor) {
-		g_dbus_remove_watch(req->conn, device->discov_listener);
-		device->discov_listener = 0;
-		g_free(device->discov_requestor);
-		device->discov_requestor = NULL;
-	}
-
-	if (req->msg)
-		dbus_message_unref(req->msg);
-	if (req->conn)
-		dbus_connection_unref(req->conn);
-	g_slist_free(req->uuids_added);
-	g_slist_free(req->uuids_removed);
-	if (req->records)
-		sdp_list_free(req->records, (sdp_free_func_t) sdp_record_free);
-	g_free(req);
-
+	browse_req_free(req);
 }
 
 static void browse_cb(sdp_list_t *recs, int err, gpointer user_data)
@@ -1086,7 +1104,7 @@ done:
 	search_cb(recs, err, user_data);
 }
 
-static void init_browse(struct browse_req *req)
+static void init_browse(struct browse_req *req, gboolean reverse)
 {
 	GSList *l;
 
@@ -1121,19 +1139,27 @@ static void init_browse(struct browse_req *req)
 		}
 	}
 
+	/* If we are doing reverse-SDP don't try to detect removed profiles
+	 * since some devices hide their service records while they are
+	 * connected
+	 */
+	if (reverse)
+		return;
+
 	for (l = req->device->uuids; l; l = l->next)
 		req->uuids_removed = g_slist_append(req->uuids_removed,
 						l->data);
 }
 
 int device_browse(struct btd_device *device, DBusConnection *conn,
-			DBusMessage *msg, uuid_t *search)
+			DBusMessage *msg, uuid_t *search, gboolean reverse)
 {
 	struct btd_adapter *adapter = device->adapter;
 	struct browse_req *req;
 	bdaddr_t src;
 	uuid_t uuid;
 	bt_callback_t cb;
+	int err;
 
 	if (device->discov_active)
 		return -EBUSY;
@@ -1152,7 +1178,7 @@ int device_browse(struct btd_device *device, DBusConnection *conn,
 		cb = search_cb;
 	} else {
 		sdp_uuid16_create(&uuid, uuid_list[req->search_uuid++]);
-		init_browse(req);
+		init_browse(req, reverse);
 		cb = browse_cb;
 	}
 
@@ -1168,8 +1194,12 @@ int device_browse(struct btd_device *device, DBusConnection *conn,
 						device, NULL);
 	}
 
-	return bt_search_service(&src, &device->bdaddr,
-					&uuid, cb, req, NULL);
+	err = bt_search_service(&src, &device->bdaddr,
+				&uuid, cb, req, NULL);
+	if (err < 0)
+		browse_req_free(req);
+
+	return err;
 }
 
 struct btd_adapter *device_get_adapter(struct btd_device *device)
@@ -1252,40 +1282,85 @@ static gboolean start_discovery(gpointer user_data)
 {
 	struct btd_device *device = user_data;
 
-	device_browse(device, NULL, NULL, NULL);
+	device_browse(device, NULL, NULL, NULL, TRUE);
 
 	device->discov_timer = 0;
 
 	return FALSE;
 }
 
-gboolean device_set_paired(DBusConnection *conn, struct btd_device *device,
+int device_set_paired(DBusConnection *conn, struct btd_device *device,
 			struct bonding_request_info *bonding)
 {
 	dbus_bool_t paired = TRUE;
 
 	device_set_temporary(device, FALSE);
 
-	dbus_connection_emit_property_changed(conn, device->path,
-						DEVICE_INTERFACE, "Paired",
-						DBUS_TYPE_BOOLEAN, &paired);
-
-	if (device->discov_timer)
-		return FALSE;
+	emit_property_changed(conn, device->path, DEVICE_INTERFACE, "Paired",
+				DBUS_TYPE_BOOLEAN, &paired);
 
 	/* If we were initiators start service discovery immediately.
 	 * However if the other end was the initator wait a few seconds
 	 * before SDP. This is due to potential IOP issues if the other
 	 * end starts doing SDP at the same time as us */
-	if (bonding)
-		device_browse(device, bonding->conn,
-				bonding->msg, NULL);
-	else
+	if (bonding) {
+		/* If we are initiators remove any discovery timer and just
+		 * start discovering services directly */
+		if (device->discov_timer) {
+			g_source_remove(device->discov_timer);
+			device->discov_timer = 0;
+		}
+
+		return device_browse(device, bonding->conn, bonding->msg,
+					NULL, FALSE);
+	}
+
+	/* If we are not initiators and there is no currently active discovery
+	 * or discovery timer, set the discovery timer */
+	if (!device->discov_active && !device->discov_timer)
 		device->discov_timer = g_timeout_add(DISCOVERY_TIMER,
 							start_discovery,
 							device);
 
-	return TRUE;
+	return 0;
+}
+
+void btd_device_add_uuid(struct btd_device *device, const char *uuid)
+{
+	GSList *uuid_list;
+	char *new_uuid;
+
+	if (g_slist_find_custom(device->uuids, uuid,
+				(GCompareFunc) strcasecmp))
+		return;
+
+	new_uuid = g_strdup(uuid);
+	uuid_list = g_slist_append(NULL, new_uuid);
+
+	device_probe_drivers(device, uuid_list);
+
+	g_free(new_uuid);
+	g_slist_free(uuid_list);
+
+	store_profiles(device);
+	services_changed(device);
+}
+
+const sdp_record_t *btd_device_get_record(struct btd_device *device,
+						const char *uuid)
+{
+	bdaddr_t src;
+
+	if (device->tmp_records)
+		return find_record_in_list(device->tmp_records, uuid);
+
+	adapter_get_address(device->adapter, &src);
+
+	device->tmp_records = read_records(&src, &device->bdaddr);
+	if (!device->tmp_records)
+		return NULL;
+
+	return find_record_in_list(device->tmp_records, uuid);
 }
 
 int btd_register_device_driver(struct btd_device_driver *driver)
